window.Gleam.initSearch([{"doc":"mist","title":"mist","content":"# mist\n\nA glistening Gleam web server.\n\n## Installation\n\nThis package can be added to your Gleam project:\n\n```sh\ngleam add mist\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/mist&gt;.\n\n## Usage\n\nRight now there are a few options.  Let&#39;s say you want a &quot;simple&quot; HTTP server\nthat you can customize to your heart&#39;s content.  In that case, you want:\n\n```gleam\nimport gleam/bit_builder\nimport gleam/erlang/process\nimport gleam/http/response\nimport mist\n\npub fn main() {\n  let assert Ok(_) =\n    mist.run_service(\n      8080,\n      fn(_req) {\n        response.new(200)\n        |&gt; response.set_body(bit_builder.from_string(&quot;hello, world!&quot;))\n      },\n      max_body_limit: 4_000_000\n    )\n  process.sleep_forever()\n}\n```\n\nMaybe you also want to work with websockets.  Maybe those should only be\nupgradable at a certain endpoint.  For that, you can use `mist.handler_func`.\nThe websocket methods help you build a handler with connect/disconnect handlers.\nYou can use these to track connected clients, for example.\n\n```gleam\nimport gleam/bit_builder\nimport gleam/erlang/process\nimport gleam/http.{Get, Post}\nimport gleam/http/request.{Request}\nimport gleam/http/response\nimport gleam/result\nimport mist\nimport mist/websocket\n\npub fn main() {\n  let assert Ok(_) =\n    mist.serve(\n      8080,\n      mist.handler_func(fn(req) {\n        case req.method, request.path_segments(req) {\n          Get, [&quot;echo&quot;, &quot;test&quot;] -&gt; websocket_echo()\n          Post, [&quot;echo&quot;, &quot;body&quot;] -&gt; echo_body(req)\n          Get, [&quot;home&quot;] -&gt;\n            response.new(200)\n            |&gt; mist.bit_builder_response(\n              bit_builder.from_string(&quot;sup home boy&quot;)\n            )\n          _, _ -&gt;\n            response.new(200)\n            |&gt; mist.bit_builder_response(\n              bit_builder.from_string(&quot;Hello, world!&quot;)\n            )\n        }\n      }),\n    )\n  process.sleep_forever()\n}\n\nfn websocket_echo() {\n  websocket.echo_handler\n  |&gt; websocket.with_handler\n  // Here you can gain access to the `Subject` to send message to\n  // with:\n  // |&gt; websocket.on_init(fn(subj) { ... })\n  // |&gt; websocket.on_close(fn(subj) { ... })\n  |&gt; mist.upgrade\n}\n\nfn echo_body(req: Request(mist.Body)) {\n  req\n  |&gt; mist.read_body\n  |&gt; result.map(fn(req) {\n    response.new(200)\n    |&gt; response.prepend_header(\n      &quot;content-type&quot;,\n      request.get_header(req, &quot;content-type&quot;)\n      |&gt; result.unwrap(&quot;application/octet-stream&quot;),\n    )\n    |&gt; mist.bit_builder_response(bit_builder.from_bit_string(req.body))\n  })\n  |&gt; result.unwrap(\n    response.new(400)\n    |&gt; mist.empty_response,\n  )\n}\n```\n\nYou might also want to use SSL.  You can do that with the following options.\n\nWith `run_service_ssl`:\n\n```gleam\nimport gleam/bit_builder\nimport gleam/erlang/process\nimport gleam/http/response\nimport mist\n\npub fn main() {\n  let assert Ok(_) =\n    mist.run_service_ssl(\n      port: 8080,\n      certfile: &quot;/path/to/server.crt&quot;,\n      keyfile: &quot;/path/to/server.key&quot;,\n      handler: fn(_req) {\n        response.new(200)\n        |&gt; response.set_body(bit_builder.from_bit_string(&lt;&lt;\n          &quot;hello, world!&quot;:utf8,\n        &gt;&gt;))\n      },\n      max_body_limit: 4_000_000\n    )\n  process.sleep_forever()\n}\n```\n\nWith `serve_ssl`:\n\n```gleam\npub fn main() {\n  let assert Ok(_) =\n    mist.serve_ssl(\n      port: 8080,\n      certfile: &quot;...&quot;,\n      keyfile: &quot;...&quot;,\n      mist.handler_func(fn(req) {\n        todo\n      }\n    )\n  // ...\n}\n```\n\nThere is support for sending files as well. This uses the `file:sendfile` erlang\nmethod under the hood.\n\n```gleam\nimport gleam/bit_builder\nimport gleam/erlang/process\nimport gleam/http/request.{Request}\nimport gleam/http/response\nimport gleam/string\nimport mist\n\npub fn main() {\n  let asset_root = &quot;...&quot;\n  let assert Ok(_) =\n    mist.serve(\n      8080,\n      mist.handler_func(fn(req: Request(Body)) {\n        let not_found =\n          response.new(404)\n          |&gt; mist.empty_response\n        case request.path_segments(req) {\n          [&quot;static&quot;, ..path] -&gt; {\n            // verify, validate, etc\n            let file_path =\n              path\n              |&gt; string.join(&quot;/&quot;)\n              |&gt; string.append(&quot;/&quot;, _)\n            response.new(200)\n            |&gt; mist.file_response(asset_root &lt;&gt; file_path)\n            |&gt; result.unwrap(not_found)\n          }\n          _ -&gt; not_found\n        }\n      }),\n    )\n  process.sleep_forever()\n}\n```\n\nYou can return chunked responses using the `mist.{chunked_response}` method.\nThis takes an `Iterator(BitBuilder)` and handles sending the initial\nresponse, and subsequent chunks in the proper format as they are emitted from\nthe iterator. The iterator must be finite.\n\nIf you need something a little more complex or custom, you can always use the\nhelpers exported by the various `glisten`/`mist` modules.\n\n## Benchmarks\n\nThese are currently located [here](https://github.com/rawhat/http-benchmarks)\n","url":"index.html"},{"doc":"mist","title":"Body","content":"pub type Body =\n  HTTPBody\n This type reflects whether the body has been read from the socket yet.\n\nSynonyms:\nmist.Body\nmist Body","url":"mist.html#Body"},{"doc":"mist","title":"Response","content":"pub type Response =\n  HandlerResponse\n Mist supports `BitBuilder`, `FileBody`, and `Chunked` response types. The\n helper methods provided can generate these from a `gleam/http/response`\n Response. This type is re-exported for pulling specific handlers out into\n separate functions.\n\nSynonyms:\nmist.Response\nmist Response","url":"mist.html#Response"},{"doc":"mist","title":"bit_builder_response","content":"pub fn bit_builder_response(resp: Response(a), data: BitBuilder) -&gt; HandlerResponse\n The mist runtime only supports sending BitBuilder types, or files (see\n below). This method will erase any pre-existing response body.\nSynonyms:\nmist.bit_builder_response\nmist bit_builder_response","url":"mist.html#bit_builder_response"},{"doc":"mist","title":"chunked_response","content":"pub fn chunked_response(resp: Response(a), iter: Iterator(\n    BitBuilder,\n  )) -&gt; HandlerResponse\n You can send chunks of responses from an iterator. The iterator must\n complete.\nSynonyms:\nmist.chunked_response\nmist chunked_response","url":"mist.html#chunked_response"},{"doc":"mist","title":"empty_response","content":"pub fn empty_response(resp: Response(a)) -&gt; HandlerResponse\n `mist.serve` expects the mist Response type, rather than the `gleam/http`\n Response. When returning a response with no body, this will convert the\n type. Note that any previously set response body will be removed before\n sending.\nSynonyms:\nmist.empty_response\nmist empty_response","url":"mist.html#empty_response"},{"doc":"mist","title":"file_response","content":"pub fn file_response(resp: Response(a), path: String, content_type: String) -&gt; Result(\n  HandlerResponse,\n  FileError,\n)\n This is a more generally optimized method for returning files to a client.\n It&#39;s a light wrapper around Erlang&#39;s `file:sendfile/5` method. The error\n can be matched on with `mist/file.{FileError}` if custom behavior is desired\n for various cases. The size of the file will be added to the\n `content-length` header field.\nSynonyms:\nmist.file_response\nmist file_response","url":"mist.html#file_response"},{"doc":"mist","title":"handler_func","content":"pub fn handler_func(handler: fn(Request(Body)) -&gt; HandlerResponse) -&gt; fn(\n  HandlerMessage,\n  LoopState(State),\n) -&gt; Next(LoopState(State))\n Handles converting the mist `Response` type into a gleam HTTP Response. Use\n this when calling `mist.serve` to start your application.\nSynonyms:\nmist.handler_func\nmist handler_func","url":"mist.html#handler_func"},{"doc":"mist","title":"read_body","content":"pub fn read_body(req: Request(Body)) -&gt; Result(\n  Request(BitString),\n  DecodeError,\n)\n When using `mist.serve`, the body is not automatically read. You can\n inspect content headers to determine whether to read the body or not.\n This function will pull the content from the sender. It gives back a\n `Request(BitString)` containing the body. This return value should be\n treated as replacing the initial request. Do not attempt to call this\n method multiple times on the same request.\nSynonyms:\nmist.read_body\nmist read_body","url":"mist.html#read_body"},{"doc":"mist","title":"run_service","content":"pub fn run_service(port: Int, handler: fn(Request(BitString)) -&gt;\n    Response(BitBuilder), max_body_limit max_body_limit: Int) -&gt; Result(\n  Nil,\n  StartError,\n)\n Runs an HTTP Request-&gt;Response server at the given port, with your defined\n handler. This will automatically read the full body contents up to the\n specified `max_body_limit` in bytes. If you&#39;d prefer to have finer-grain\n control over this behavior, consider using `mist.serve`.\nSynonyms:\nmist.run_service\nmist run_service","url":"mist.html#run_service"},{"doc":"mist","title":"run_service_ssl","content":"pub fn run_service_ssl(port port: Int, certfile certfile: String, keyfile keyfile: String, handler handler: fn(\n    Request(BitString),\n  ) -&gt; Response(BitBuilder), max_body_limit max_body_limit: Int) -&gt; Result(\n  Nil,\n  StartError,\n)\n Similar setup and behavior to `run_service`, but instead takes in the SSL\n certificate/key and serves over HTTPS.\nSynonyms:\nmist.run_service_ssl\nmist run_service_ssl","url":"mist.html#run_service_ssl"},{"doc":"mist","title":"serve","content":"pub fn serve(port port: Int, handler handler: fn(\n    HandlerMessage,\n    LoopState(State),\n  ) -&gt; Next(LoopState(State))) -&gt; Result(Nil, StartError)\n Slightly more flexible alternative to `run_service`. This allows hooking\n into the `mist.handler_func` method. Note that the request body\n will not be automatically read. You will need to call `mist.read_body`.\n Ensure that this is only called _once_ per request.\nSynonyms:\nmist.serve\nmist serve","url":"mist.html#serve"},{"doc":"mist","title":"serve_ssl","content":"pub fn serve_ssl(port port: Int, certfile certfile: String, keyfile keyfile: String, handler handler: fn(\n    HandlerMessage,\n    LoopState(State),\n  ) -&gt; Next(LoopState(State))) -&gt; Result(Nil, StartError)\n Similar to the `run_service` method, `serve` also has a similar SSL method.\nSynonyms:\nmist.serve_ssl\nmist serve_ssl","url":"mist.html#serve_ssl"},{"doc":"mist","title":"upgrade","content":"pub fn upgrade(websocket_handler: WebsocketHandler) -&gt; HandlerResponse\n A websocket handler is created using the `websocket.with_handler`\n method. This function enables the mist HTTP layer to build the properly\n formatted websocket upgrade response.\nSynonyms:\nmist.upgrade\nmist upgrade","url":"mist.html#upgrade"},{"doc":"mist","title":"mist","content":"","url":"mist.html"},{"doc":"mist/websocket","title":"echo_handler","content":"pub fn echo_handler(message: Message, sender: Subject(\n    HandlerMessage,\n  )) -&gt; Result(Nil, Nil)\n\nSynonyms:\nmist/websocket.echo_handler\nmist/websocket echo_handler","url":"mist/websocket.html#echo_handler"},{"doc":"mist/websocket","title":"on_close","content":"pub fn on_close(handler: WebsocketHandler, func: fn(\n    Subject(HandlerMessage),\n  ) -&gt; Nil) -&gt; WebsocketHandler\n\nSynonyms:\nmist/websocket.on_close\nmist/websocket on_close","url":"mist/websocket.html#on_close"},{"doc":"mist/websocket","title":"on_init","content":"pub fn on_init(handler: WebsocketHandler, func: fn(\n    Subject(HandlerMessage),\n  ) -&gt; Nil) -&gt; WebsocketHandler\n\nSynonyms:\nmist/websocket.on_init\nmist/websocket on_init","url":"mist/websocket.html#on_init"},{"doc":"mist/websocket","title":"send","content":"pub fn send(sender: Subject(HandlerMessage), message: Message) -&gt; Nil\n Helper to encapsulate the logic to send a provided message over the\n WebSocket\nSynonyms:\nmist/websocket.send\nmist/websocket send","url":"mist/websocket.html#send"},{"doc":"mist/websocket","title":"with_handler","content":"pub fn with_handler(func: fn(Message, Subject(HandlerMessage)) -&gt;\n    Result(Nil, Nil)) -&gt; WebsocketHandler\n\nSynonyms:\nmist/websocket.with_handler\nmist/websocket with_handler","url":"mist/websocket.html#with_handler"},{"doc":"mist/websocket","title":"mist/websocket","content":"","url":"mist/websocket.html"}]);